@@ -1,6 +1,6 @@
 from hapiclient import autoplot
 
-server     = 'http://hapi-server.org/servers/TestData/hapi'
+server     = 'http://hapi-server.org/servers/TestData2.0/hapi'
 dataset    = 'dataset1'
 start      = '1970-01-01Z'
 stop       = '1970-01-01T00:00:11Z'
