@@ -295,9 +295,9 @@ def heatmap(x, y, z, **kwargs):
         FigureCanvas(fig) # Not used directly, but calling attaches canvas to fig which is used later.
         ax = fig.add_subplot(111)
     else:
-        #plt.figure()
-        fig = plt.gcf()
-        ax = plt.gca()
+        fig, ax = plt.subplots()
+        if opts['logging']:
+            print("timeseries(): Using Matplotlib back-end " + matplotlib.get_backend())
 
     if type(x) == list:
         x = np.array(x)
@@ -363,8 +363,11 @@ def heatmap(x, y, z, **kwargs):
 
     inan = np.where(np.isnan(z))
     havenans = False
+    allnan = False
     if np.any(inan) > 0:
         havenans = True
+    if np.all(np.isnan(z)):
+        allnan = True
 
     xgaps = np.array([], dtype=np.int32)
     ygaps = np.array([], dtype=np.int32)
@@ -399,74 +402,26 @@ def heatmap(x, y, z, **kwargs):
                                      edgecolor=opts['edgecolor'], label='No data'))
 
     if havenans:
+        edgecolor = opts['edgecolor']
+        if allnan and edgecolor is None:
+            edgecolor = 'k'
         cmapn = colors.LinearSegmentedColormap.from_list('nan',[opts['nan.color'],opts['nan.color']],2)        
         zn = np.nan*np.copy(z)
         if havegaps:
             inan = np.where(np.logical_and(np.isnan(z),zg != 1))
         zn[inan] = 1
+
         if opts['nan.hatch'] == '':
-            ax.pcolormesh(x, y, zn, cmap=cmapn)
+            ax.pcolormesh(x, y, zn, edgecolor=edgecolor, cmap=cmapn)
         else:
             # Must set hatch.color through rc params and context manager.
             with rc_context(rc={'hatch.color': opts['nan.hatch.color']}):
-                ax.pcolor(x, y, zn, cmap=cmapn, hatch=opts['nan.hatch'])
+                ax.pcolor(x, y, zn, cmap=cmapn, edgecolor=edgecolor, hatch=opts['nan.hatch'])
         if opts['nan.legend']:
             with rc_context(rc={'hatch.color': opts['nan.hatch.color']}):
                 legendh.append(Patch(facecolor=opts['nan.color'],
                                      hatch=opts['nan.hatch']+opts['nan.hatch'],
-                                     edgecolor=opts['edgecolor'], label='NaN'))
-
-    
-    zc = np.array([])   
-    allintz = allint(z)
-    if not 'cmap' in kwargs and allintz:
-        Ig = ~np.isnan(z)
-        zc = np.unique(z[Ig])
-        nc = zc[-1]-zc[0] + 1
-        nc = np.min([1024, nc])
-        if 'cmap.numcolors' in kwargs:
-            if opts['cmap.numcolors'] != nc:
-                warning('Overriding requested number of colors. Using number of colors = ' + str(nc))
-        cmap_name = opts['cmap.name']
-        if len(Ig) == 2 and not 'cmap.name' in kwargs:
-            # Binary. Plot black and white.
-            cmap_name = 'gray'
-        opts['cmap'] = matplotlib.pyplot.get_cmap(cmap_name, nc)
-
-    zmin = np.nanmin(z)
-    zmax = np.nanmax(z)
-    if zmin < 0 and zmax > 0:
-        warning('Colorbar cannot have log scale when all values do not have the same sign.')
-
-    havelogz0 = False
-    if opts['logz']:
-        # Log scale emits warning if data have NaNs.
-        warnings.filterwarnings(action='ignore',
-                            message='invalid value encountered in less_equal')
-        flipsign = False
-        if zmin < 0 and zmax <= 0 and opts['logz']:
-            z = -z;
-            flipsign = True
-        if zmin == 0:
-            #warning('Log scale for z requested but min(z) = 0.')
-            havelogz0 = True
-            logz0idx = np.where(z==0)
-            z[z == 0] = np.nan
-            havelogz0 = True
-            zmin = np.nanmin(z)
-        norm = LogNorm(vmin=zmin, vmax=zmax)
-        im = ax.pcolormesh(x, y, z, cmap=opts['cmap'], norm=norm,
-                           edgecolor=opts['edgecolor'])
-        if havelogz0:
-            cmapz = colors.LinearSegmentedColormap.from_list('logz0',[opts['logz0.color'],opts['logz0.color']],2)
-            z = np.nan*z
-            z[logz0idx] = 1
-            ax.pcolormesh(x, y, z, cmap=cmapz)
-            if opts['logz0.legend']:
-                legendh.append(Patch(facecolor=opts['logz0.color'],
-                                     edgecolor='k', label='0.0'))
-    else:
-        im = ax.pcolormesh(x, y, z, cmap=opts['cmap'], edgecolor=opts['edgecolor'])
+                                     edgecolor=edgecolor, label='NaN'))
 
     # TODO: Handle case where > 10.
     # Label every Nth, etc. as needed
@@ -531,8 +486,6 @@ def heatmap(x, y, z, **kwargs):
     if opts['title']:
         ax.set_title(opts['title'], fontsize=10)
 
-    cb = fig.colorbar(im, ax=ax, pad=0.01)
-
     if False and allint(x):
         xa = ax.get_yaxis()
         xa.set_major_locator(MaxNLocator(integer=True, steps=[1, 2, 4, 5, 10]))
@@ -540,41 +493,107 @@ def heatmap(x, y, z, **kwargs):
     if False and allint(y):
         ya = ax.get_yaxis()
         ya.set_major_locator(MaxNLocator(integer=True, steps=[1, 2, 4, 5, 10]))
-        
-    if allintz:
-        # Put tick label at center of color patch.
-        if not opts['cmap.clim']:
-            im.set_clim(zc[0]-0.5, zc[-1] + 0.5)
-        za = cb.ax.get_yaxis()
-        za.set_major_locator(MaxNLocator(integer=True, min_n_ticks=1, steps=[1, 2, 4, 5, 10]))
-    if opts['cmap.clim']:
-        im.set_clim(opts['cmap.clim'])
 
-    zt = cb.get_ticks()
+    # NaN, logz0, and gap legend
+    if len(legendh) > 0:
+        fig.legend(frameon=False, borderaxespad=0.25, borderpad=0.15,
+                   handles=legendh, loc='upper right',
+                   fontsize='x-small', ncol=len(legendh))
 
-    # How to label zmin and zmax without overlap of default labels?
-    # Extend clim to next interval? But then one may think actual values in
-    # extended range. For now, just warn.
-    if zt[0] != zmin:
-        #warnings.warn('Lower z limit not labeled.')
-        zt = np.concatenate(([zmin],zt))
-        #cb.set_ticks(zt)
-    if zt[-1] != zmax:
-        #warnings.warn('Upper z limit not labeled.')
-        zt = np.concatenate((zt,[zmax]))
-        #cb.set_ticks(zt)
+    if allnan:
+        cb = None
+    else:
+        # Colorbar
+        zc = np.array([])
+        allintz = allint(z)
+        if not 'cmap' in kwargs and allintz:
+            Ig = ~np.isnan(z)
+            zc = np.unique(z[Ig])
+            if len(zc) == 1:
+                nc = len(zc)
+            else:
+                nc = np.int(zc[-1]-zc[0] + 1)
+            nc = np.min([1024, nc])
+            if 'cmap.numcolors' in kwargs:
+                if opts['cmap.numcolors'] != nc:
+                    warning('Overriding requested number of colors. Using number of colors = ' + str(nc))
+            cmap_name = opts['cmap.name']
+            if len(Ig) == 2 and not 'cmap.name' in kwargs:
+                # Binary. Plot black and white.
+                cmap_name = 'gray'
+            opts['cmap'] = matplotlib.pyplot.get_cmap(cmap_name, nc)
 
-    cb.set_label(opts['zlabel'], fontsize=10) # On side
-    cb.ax.set_title(opts['ztitle'], fontsize=10) # On top
+        zmin = np.nanmin(z)
+        zmax = np.nanmax(z)
+        if zmin < 0 and zmax > 0:
+            warning('Colorbar cannot have log scale when all values do not have the same sign.')
 
-    if opts['logz'] and flipsign:
-        # Put negative sign in front of numbers on colorbar
-        zlabels = cb.ax.get_yticklabels()
-        for i in range(0, len(zlabels)):
-            text = zlabels[i].get_text()
-            if text != '':
-                zlabels[i].set_text('-'+text)
-        cb.ax.set_yticklabels(zlabels)
+        havelogz0 = False
+        if not opts['logz']:
+            im = ax.pcolormesh(x, y, z, cmap=opts['cmap'], edgecolor=opts['edgecolor'])
+        else:
+            # Log scale emits warning if data have NaNs.
+            warnings.filterwarnings(action='ignore',
+                                message='invalid value encountered in less_equal')
+            flipsign = False
+            if zmin < 0 and zmax <= 0 and opts['logz']:
+                z = -z;
+                flipsign = True
+            if zmin == 0:
+                #warning('Log scale for z requested but min(z) = 0.')
+                havelogz0 = True
+                logz0idx = np.where(z==0)
+                z[z == 0] = np.nan
+                havelogz0 = True
+                zmin = np.nanmin(z)
+            norm = LogNorm(vmin=zmin, vmax=zmax)
+            im = ax.pcolormesh(x, y, z, cmap=opts['cmap'], norm=norm,
+                               edgecolor=opts['edgecolor'])
+            if havelogz0:
+                cmapz = colors.LinearSegmentedColormap.from_list('logz0',[opts['logz0.color'],opts['logz0.color']],2)
+                z = np.nan*z
+                z[logz0idx] = 1
+                ax.pcolormesh(x, y, z, cmap=cmapz)
+                if opts['logz0.legend']:
+                    legendh.append(Patch(facecolor=opts['logz0.color'],
+                                         edgecolor='k', label='0.0'))
+
+        cb = fig.colorbar(im, ax=ax, pad=0.01)
+
+        if allintz:
+            # Put tick label at center of color patch.
+            if not opts['cmap.clim']:
+                im.set_clim(zc[0]-0.5, zc[-1] + 0.5)
+            za = cb.ax.get_yaxis()
+            za.set_major_locator(MaxNLocator(integer=True, min_n_ticks=1, steps=[1, 2, 4, 5, 10]))
+        if opts['cmap.clim']:
+            im.set_clim(opts['cmap.clim'])
+
+        zt = cb.get_ticks()
+
+        # How to label zmin and zmax without overlap of default labels?
+        # Extend clim to next interval? But then one may think actual values in
+        # extended range. For now, just warn.
+        if zt[0] != zmin:
+            #warnings.warn('Lower z limit not labeled.')
+            zt = np.concatenate(([zmin],zt))
+            #cb.set_ticks(zt)
+        if zt[-1] != zmax:
+            #warnings.warn('Upper z limit not labeled.')
+            zt = np.concatenate((zt,[zmax]))
+            #cb.set_ticks(zt)
+
+        cb.set_label(opts['zlabel'], fontsize=10) # On side
+        cb.ax.set_title(opts['ztitle'], fontsize=10) # On top
+
+        if opts['logz'] and flipsign:
+            # Put negative sign in front of numbers on colorbar
+            zlabels = cb.ax.get_yticklabels()
+            for i in range(0, len(zlabels)):
+                text = zlabels[i].get_text()
+                if text != '':
+                    zlabels[i].set_text('-'+text)
+            cb.ax.set_yticklabels(zlabels)
 
     if isinstance(x[0], datetime.datetime):
         datetick('x', axes=ax)
@@ -600,20 +619,14 @@ def heatmap(x, y, z, **kwargs):
     # Set positions of axes and colorbar
     ax_x = 0.1   # Offset from left
     ax_y = 0.14  # Offset from bottom
-    ax_h = ax_h  # Height
     cb_x = ax_x+ax_w+0.005 # Offset from left
     cb_y = ax_y  # Offset from bottom
     cb_w = 0.1   # Width
     cb_h = ax_h  # Height
-    
-    ax.set_position([ax_x,ax_y,ax_w,ax_h])
-    cb.ax.set_position([cb_x,cb_y,cb_w,cb_h])
 
-    # NaN, logz0, and gap legend
-    if len(legendh) > 0:
-        fig.legend(frameon=False, borderaxespad=0.25, borderpad=0.15,
-                   handles=legendh, loc='upper right',
-                   fontsize='x-small', ncol=len(legendh))
+    ax.set_position([ax_x,ax_y,ax_w,ax_h])
+    if not allnan:
+        cb.ax.set_position([cb_x,cb_y,cb_w,cb_h])
 
     if opts['transparent']:
         fig.patch.set_alpha(0)
