@@ -25,7 +25,6 @@ def timeseries(t, y, **kwargs):
                 'ylabel': '',
                 'logx': '',
                 'logy': '',
-                'backend': 'default',
                 'returnimage': False,
                 'transparent': False,
                 'legendlabels': []
@@ -36,7 +35,7 @@ def timeseries(t, y, **kwargs):
             opts[key] = value
         else:
             warnings.warn('Warning: Ignoring invalid keyword option "%s".' % key, SyntaxWarning)
-    
+
     if opts['returnimage']:
         # When returnimage=True, the Matplotlib OO API is used b/c it is thread safe.
         # Otherwise, the pyplot API is used. Ideally would always use the OO API,
@@ -50,7 +49,8 @@ def timeseries(t, y, **kwargs):
         from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
     else:
         from matplotlib import pyplot as plt
-        if opts['logging']: print("timeseries(): Using Matplotlib back-end " + matplotlib.get_backend())
+        if opts['logging']:
+            print("timeseries(): Using Matplotlib back-end " + matplotlib.get_backend())
 
     if y.shape[0] < 11:
         props = {'linestyle': 'none', 'marker': '.', 'markersize': 16}
@@ -79,6 +79,16 @@ def timeseries(t, y, **kwargs):
         ylabels = yu
         y = yi
 
+    all_nan = False
+    if np.all(np.isnan(y)):
+        all_nan = True
+        if len(y) > 1:
+            legendlabels = ['All %d values are NaN' % len(y)]
+        if len(y) == 1:
+            legendlabels = ['One NaN value in time interval']
+    else:
+        legendlabels = opts['legendlabels']
+ 
     # Can't use matplotlib.style.use(style) because not thread safe.
     # Set context using 'with'.
     # Setting stylesheet method: https://stackoverflow.com/a/22794651/1491619
@@ -86,40 +96,31 @@ def timeseries(t, y, **kwargs):
         # See note above about OO API for explanation for why this is
         # done differently if returnimage=True
         fig = Figure()
-        FigureCanvas(fig) # Not used directly, but calling attaches canvas to fig which is needed by datetick and hapiplot.
+        # Attach canvas to fig, which is needed by datetick and hapiplot.
+        FigureCanvas(fig) 
         ax = fig.add_subplot(111)
-        ax.plot(t, y, **props)
-        ax.set(ylabel=opts['ylabel'], xlabel=opts['xlabel'], title=opts['title'])
-        try:
-            ax.ticklabel_format(axis='y', style='sci', scilimits=(-3,3))
-        except:
-            pass
-        ax.set_position([0.12,0.125,0.850,0.75])
-        if opts['legendlabels'] != []:
-            ax.legend(opts['legendlabels'])
     else:
+        fig, ax = plt.subplots()
+        fig.canvas.set_window_title(opts['title'])
 
-        # Needed with Qt5 back-end. With Tk back-end, causes an extra empty window.
-        plt.figure() 
-        plt.plot(t, y, **props)
-        plt.gcf().canvas.set_window_title(opts['title'])
-        plt.ylabel(opts['ylabel'])
-        plt.xlabel(opts['xlabel'])
-        plt.title(opts['title'])
+    if all_nan:
+        ax.plot([t[0], t[-1]], [0, 0], linestyle='None')
+        plt.gca().set_yticklabels([])
+    else:
+        ax.plot(t, y, **props)
 
-        if opts['legendlabels'] != []:
-            plt.legend(opts['legendlabels'])
-        ax = plt.gca()
-        ax.set_position([0.12,0.125,0.850,0.75])
-        try:
-            ax.ticklabel_format(axis='y', style='sci', scilimits=(-3,3))
-        except:
-            pass
-        fig = plt.gcf()
+    ax.set(ylabel=opts['ylabel'], xlabel=opts['xlabel'], title=opts['title'])
+    try:
+        ax.ticklabel_format(axis='y', style='sci', scilimits=(-3,3))
+    except:
+        pass
+    if not all_nan and legendlabels != []:
+        fig.legend(opts['legendlabels'])
+    ax.set_position([0.12,0.125,0.850,0.75])
 
     ax.grid()
 
-    if len(ylabels) > 0:
+    if not all_nan and len(ylabels) > 0:
         ax.set_yticks(np.unique(y))
         ax.set_yticklabels(ylabels)
 
@@ -127,17 +128,7 @@ def timeseries(t, y, **kwargs):
         datetick('x', axes=ax)
     if isinstance(y[0], datetime.datetime):
         datetick('y', axes=ax)
-    
-    # See
-    # https://stackoverflow.com/questions/24581194/matplotlib-text-bounding-box-dimensions
-    # for determining text bounding box in figure coordinates
-    if False:
-        for item in ax.get_yticklabels():
-            ml = 0 # max length
-            for t in item.get_text().split('\n'):
-                l = len(t)
-                if l > ml: ml = l 
-    
+        
     # savefig.transparent=True requires the following for the saved image
     # to have a transparent background. Seems as though figure.facealpha
     # and axes.facealpha should be rc parameters, but they are not. So
@@ -151,3 +142,13 @@ def timeseries(t, y, **kwargs):
         plt.show()
 
     return fig
+
+def adjust_labels(ax):
+    # Not used. See
+    # https://stackoverflow.com/questions/24581194/matplotlib-text-bounding-box-dimensions
+    # for determining text bounding box in figure coordinates
+    for item in ax.get_yticklabels():
+        ml = 0 # max length
+        for t in item.get_text().split('\n'):
+            l = len(t)
+            if l > ml: ml = l 
