@@ -1,18 +1,21 @@
-import pytest
 import os
+import pytest
 import pickle
+import shutil
+
 import numpy as np
 
 from deepdiff import DeepDiff
 from hapiclient.hapi import hapi
-from hapiclient.test.readcompare import readcompare, clearcache
+from hapiclient.test.readcompare import readcompare
 
 serverbad = 'http://hapi-server.org/servers/TestData/xhapi'
 server = 'http://hapi-server.org/servers/TestData2.0/hapi'
 
-# To use in program, use, e.g.,
-# from hapiclient.test.test_hapi import test_reader_short
-# test_reader_short()
+# To run the long benchmark, execute
+#    python test_hapi.py
+
+# See comments in test_hapitime2datetime.py for other execution options.
 
 def writepickle(fname, var):
     print("!!!!!!!!!!!!!!")
@@ -22,25 +25,34 @@ def writepickle(fname, var):
         pickle.dump(var, pickle_file, protocol=2)
     pickle_file.close()
 
+
 def readpickle(fname):
     with open(fname, 'rb') as pickle_file:
         var = pickle.load(pickle_file)
     pickle_file.close()
     return var
 
+
+@pytest.mark.short
 def test_hapi():
     """Test that a call with no parameters returns something."""
     assert hapi() is not None
 
+
+@pytest.mark.short
 def test_server_list():
     """Test that specifying a server returns something."""
     assert hapi(server) is not None
 
+
+@pytest.mark.short
 def test_catalog():
     """Request for catalog returns correct status and first dataset"""
     meta = hapi(server)
     assert meta['status'] == {'code': 1200, 'message': 'OK'} and meta['catalog'][0]['id'] == 'dataset1'
 
+
+@pytest.mark.short
 def test_dataset():
     """Request for dataset returns correct dataset metadata"""
     meta = hapi(server,'dataset1')
@@ -54,6 +66,8 @@ def test_dataset():
         metatest = readpickle(pklFile)
     assert DeepDiff(meta,metatest) == {}
 
+
+@pytest.mark.short
 def test_parameter():
     """Request for dataset returns correct parameter metadata"""
     meta = hapi(server,'dataset1')
@@ -67,60 +81,75 @@ def test_parameter():
         metatest = readpickle(pklFile)
     assert DeepDiff(meta,metatest) == {}
  
+
+@pytest.mark.short
 def test_bad_server_url():
     """Correct error when given bad URL"""
     with pytest.raises(Exception):
         hapi(serverbad, {'logging': True})
 
+
+@pytest.mark.short
 def test_bad_dataset_name():
     """Correct error when given nonexistent dataset name"""
     with pytest.raises(Exception):
         hapi(server,'dataset1x')
 
+
+@pytest.mark.short
 def test_bad_parameter():
     """Correct error when given nonexistent parameter name"""
     with pytest.raises(Exception):
         hapi(server,'dataset1','scalarx')
 
+
+@pytest.mark.short
 def test_reader_short():
         
     dataset = 'dataset1'
     run = 'short'
     
+    #
     # Cache = False (will read data into buffer)    
+    #
     opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'usecache': False}
 
     opts['cache'] = False
 
     # Read one parameter
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, 'scalar', run, opts)
 
     # Read two parameters
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, 'scalar,vector', run, opts)
 
     # Read all parameters
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, '', run, opts)
 
+    #
     # Cache = True (will write files then read)
+    #
     opts['cache'] = True
 
     # Read one parameter
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, 'scalar', run, opts)
 
     # Read two parameters
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, 'scalar,vector', run, opts)
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
 
     # Read all parameters
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     assert readcompare(server, dataset, '', run, opts)
-    
+
+
+@pytest.mark.short
 def test_cache_short():
+
     # Compare read with empty cache with read with hot cache and usecache=True
     dataset = 'dataset1'
     start = '1970-01-01'
@@ -129,7 +158,7 @@ def test_cache_short():
     opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'cache': True}
 
     opts['usecache'] = False
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data, meta  = hapi(server, dataset, 'scalarint,vectorint', start, stop, **opts)
 
     opts['usecache'] = True
@@ -137,6 +166,8 @@ def test_cache_short():
 
     assert np.array_equal(data, data2)
 
+
+@pytest.mark.short
 def test_subset_short():
     
     dataset = 'dataset1'
@@ -147,10 +178,10 @@ def test_subset_short():
     opts['usecache'] = False
     
     # Request two subsets with empty cache. Common parts should be same.
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data, meta  = hapi(server, dataset, 'scalarint', start, stop, **opts)
 
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data2, meta2  = hapi(server, dataset, 'scalarint,vectorint', start, stop, **opts)
 
     ok = np.array_equal(data['Time'], data2['Time'])
@@ -158,19 +189,21 @@ def test_subset_short():
     assert ok
 
     # Request all parameters and single parameter. Common parameter should be same.
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data, meta  = hapi(server, dataset, '', start, stop, **opts)
 
-    clearcache(opts)
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data2, meta2  = hapi(server, dataset, 'vectorint', start, stop, **opts)
         
     ok = np.array_equal(data['Time'], data2['Time'])
     ok = ok and np.array_equal(data['vectorint'], data2['vectorint'])
     assert ok
 
+
     opts['usecache'] = True
     
-    # Request two subsets with hot cache. Common parts should be same.
+    # Request two subsets, with the second request using the cache. Common
+    # parts should be same.
     data, meta  = hapi(server, dataset, 'scalarint', start, stop, **opts)
     data2, meta2  = hapi(server, dataset, 'scalarint,vectorint', start, stop, **opts)
 
@@ -178,26 +211,33 @@ def test_subset_short():
     ok = ok and np.array_equal(data['scalarint'], data2['scalarint'])
     assert ok
 
-    # Request all parameters and single parameter with hot cache. Common parameter should be same.
-    clearcache(opts)
+    # Request all parameters and single parameter, with the single parameter
+    # request using the cache with hot cache. Common parameter should be same.
+    shutil.rmtree(opts['cachedir'], ignore_errors=True)
     data, meta  = hapi(server, dataset, '', start, stop, **opts)
-
-    clearcache(opts)
     data2, meta2  = hapi(server, dataset, 'vectorint', start, stop, **opts)
         
     ok = np.array_equal(data['Time'], data2['Time'])
     ok = ok and np.array_equal(data['vectorint'], data2['vectorint'])
     assert ok
 
+
 @pytest.mark.long
 def test_reader_long():
     
-    opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'cache': False, 'usecache': False}
     dataset = 'dataset1'
-    
     run = 'long'
     
-    # Read two parameters
-    clearcache(opts)
-    assert readcompare(server, dataset, 'scalar,vector', run, opts)
-        
\ No newline at end of file
+    # Read three parameters
+    opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'cache': False, 'usecache': False}
+    assert readcompare(server, dataset, 'scalar,vector,spectra', run, opts)
+
+    opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'cache': True, 'usecache': False}
+    assert readcompare(server, dataset, 'scalar,vector,spectra', run, opts)
+
+    opts = {'logging': False, 'cachedir': '/tmp/hapi-data', 'cache': False, 'usecache': True}
+    assert readcompare(server, dataset, 'scalar,vector,spectra', run, opts)
+
+
+if __name__ == '__main__':
+    test_reader_long()        
\ No newline at end of file
